@@ -74,9 +74,9 @@ class GeneticAlgorithm2:
 
     def __init__(
         self,
-        function: FunctionToMinimize,
+        function: FunctionToMinimize = None,
 
-        dimension: int,
+        dimension: int = 0,
         variable_type: Union[VARIABLE_TYPE, Sequence[VARIABLE_TYPE]] = 'bool',
         variable_boundaries: Optional[Union[array2D, Sequence[Tuple[float, float]]]] = None,
 
@@ -89,8 +89,8 @@ class GeneticAlgorithm2:
         initializes the GA object and performs main checks
 
         Args:
-            function: the given objective function to be minimized
-            dimension: the number of decision variables, the population samples dimention
+            function: the given objective function to be minimized -- deprecated and moved to run() method
+            dimension: the number of decision variables, the population samples dimension
 
             variable_type: string means the variable type for all variables,
                 for mixed types use sequence of strings of type for each variable
@@ -108,7 +108,7 @@ class GeneticAlgorithm2:
             function_timeout: if the given function does not provide
                 output before function_timeout (unit is seconds) the algorithm raises error.
                 For example, when there is an infinite loop in the given function.
-                `None` means disabling
+                `None` means disabling -- deprecated and moved to run()
 
             algorithm_parameters: AlgorithmParams object or usual dictionary with algorithm parameter;
                 it is not mandatory to provide all possible parameters
@@ -118,9 +118,6 @@ class GeneticAlgorithm2:
             For maximization u can multiply the function by -1 (for instance): the absolute
                 value of the output would be the actual objective function
 
-            - If u want to use set_function only and maybe u dont have usual function,
-                just set the function to something like lambda x: 0 but set function_timeout=None too
-
         for more details and examples of implementation please visit:
             https://github.com/PasaOpasen/geneticalgorithm2
   
@@ -133,12 +130,17 @@ class GeneticAlgorithm2:
         # self.discrete_mutation: Callable[[int, int, int], int] = None
         # self.selection: Callable[[np.ndarray, int], np.ndarray] = None
 
+        self.result = None
+        self.best_function = None
+
         self.revolution_oppositor = None
         self.dup_oppositor = None
         self.creator = None
         self.init_oppositors = None
+
         self.f: Callable[[array1D], float] = None
         self.funtimeout: float = None
+
         self.set_function: Callable[[np.ndarray], np.ndarray] = None
 
         # self.dim: int = None
@@ -191,20 +193,18 @@ class GeneticAlgorithm2:
 
         #############################################################
         # input function
-        assert (callable(function)), "function must be callable!"
-        self.f = function
-
-        if function_timeout is not None and function_timeout > 0:
-            try:
-                from func_timeout import func_timeout, FunctionTimedOut
-            except ModuleNotFoundError:
-                raise ModuleNotFoundError(
-                    "function_timeout > 0 needs additional package func_timeout\n"
-                    "run `python -m pip install func_timeout`\n"
-                    "or disable this parameter: function_timeout=None"
-                )
-
-        self.funtimeout = None if function_timeout is None else float(function_timeout)
+        if function:
+            warnings.warn(
+                f"function is deprecated in init constructor and will be removed in version 7. "
+                f"Move this argument to run() method"
+            )
+            self._check_function(function)
+        if function_timeout:
+            warnings.warn(
+                f"function_timeout is deprecated in init constructor and will be removed in version 7. "
+                f"Move this argument to run() method"
+            )
+            self._check_function_timeout(function_timeout)
 
         #############################################################
         
@@ -325,6 +325,24 @@ class GeneticAlgorithm2:
         else:
             self.max_stagnations = math.ceil(max_it)
 
+    def _check_function(self, function: Callable[[array1D], float]):
+        assert callable(function), "function must be callable!"
+        self.f = function
+
+    def _check_function_timeout(self, function_timeout: Optional[float]):
+
+        if function_timeout is not None and function_timeout > 0:
+            try:
+                from func_timeout import func_timeout, FunctionTimedOut
+            except ModuleNotFoundError:
+                raise ModuleNotFoundError(
+                    "function_timeout > 0 needs additional package func_timeout\n"
+                    "run `python -m pip install func_timeout`\n"
+                    "or disable this parameter: function_timeout=None"
+                )
+
+        self.funtimeout = None if function_timeout is None else float(function_timeout)
+
     #endregion
 
     #region REPORT
@@ -432,6 +450,9 @@ class GeneticAlgorithm2:
         # deprecated
         disable_progress_bar: bool = False,
 
+        function: FunctionToMinimize = None,
+        function_timeout: Optional[float] = None,
+
         set_function: SetFunctionToMinimize = None,
         apply_function_to_parents: bool = False,
         start_generation: GenerationConvertible = Generation(),
@@ -458,22 +479,32 @@ class GeneticAlgorithm2:
         time_limit_secs: Optional[float] = None,
         save_last_generation_as: Optional[str] = None,
         seed: Optional[int] = None
-    ):
+    ) -> GAResult:
         """
         runs optimization process
 
         Args:
             no_plot: do not plot results using matplotlib by default
 
-            disable_printing: do not print log info of optimization process
+            disable_printing: do not print log info of optimization process (except progress bar)
 
-            progress_bar_stream: 'stdout', 'stderr' or None to disable progress bar
+            progress_bar_stream: 'stdout', 'stderr' or None to disable progress bar;
+                disabling progress bar can speed up the optimization process in many cases
 
             disable_progress_bar: deprecated
 
-            set_function: set function to be used instead of usual function
+            function: the given objective function (sample -> its score) to be minimized;
 
-            apply_function_to_parents: whether to apply function to parents from previous generation (if it's needed)
+            function_timeout: if the given function does not provide
+                output before function_timeout (unit is seconds) the algorithm raises error.
+                For example, when there is an infinite loop in the given function.
+                `None` means disabling
+
+            set_function: set function (all samples -> score per sample) to be used instead of usual function
+                (usually for optimization purposes)
+
+            apply_function_to_parents: whether to apply function to parents from previous generation (if it's needed), 
+                it can be needed at working with games agents, but for other tasks will just waste time
 
             start_generation: initial generation object of any `GenerationConvertible` type
 
@@ -497,19 +528,32 @@ class GeneticAlgorithm2:
             population_initializer: object for actions at population initialization step
                 to create better start population. See doc
 
-            stop_when_reached: stop searching after reaching this value (it can be potential minimum or something else)
+            stop_when_reached: stop searching after reaching this value 
+                (it can be potential minimum or something else)
 
             callbacks: sequence of callback functions with structure:
                 (generation_number, report_list, last_population, last_scores) -> do some action
 
-            middle_callbacks: sequence of functions made `MiddleCallback` class
+            middle_callbacks: sequence of functions made by `MiddleCallback` class
 
-            time_limit_secs: limit time of working (in seconds)
+            time_limit_secs: limit time of working (in seconds);
+                `None` means no time limit (limit will be only for count of generation and so on)
 
-            save_last_generation_as: path to .npz file for saving last_generation as numpy dictionary like
-                {'population': 2D-array, 'scores': 1D-array}, None if doesn't need to save in file
+            save_last_generation_as: path to .npz file 
+                for saving last_generation as numpy dictionary like
+                {'population': 2D-array, 'scores': 1D-array}; 
+                None disables this option
 
             seed: random seed (None if doesn't matter)
+
+        Returns:
+            `GAResult` object;
+            also fills the self.report and self.result with many report information
+
+        Notes:
+            - if `function_timeout` is enabled then `function` must be set
+            - it would be more logical to use params like `studEA` as an algorithm parameter, 
+                but `run()`-way can be more convenient for real using
         """
 
         if disable_progress_bar:
@@ -644,7 +688,7 @@ class GeneticAlgorithm2:
         else:
             def total_middle_callback():
                 """
-                applies callbacks and sets new data if there is a sence
+                applies callbacks and sets new data if there is a sense
                 """
                 data = get_data()
                 flag = False
@@ -664,6 +708,16 @@ class GeneticAlgorithm2:
             else (lambda: int(time.time() - start_time) >= time_limit_secs)
         )
 
+        # combine with deprecated parts
+        function = function or self.f
+        function_timeout = function_timeout or self.funtimeout
+
+        assert function or set_function, 'no function to minimize'
+        if function:
+            self._check_function(function)
+        if function_timeout:
+            self._check_function_timeout(function_timeout)
+
         self.set_function = set_function or GeneticAlgorithm2.default_set_function(self.f)
 
         #region Initial population, duplicates filter, revolutionary
